
import net.htmlparser.jericho.*;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.io.*;
import java.net.*;


public class htmlfilterScrubber {
	
	String[] w = {HTMLElementName.BR,
		HTMLElementName.P,
		HTMLElementName.B,
		HTMLElementName.I,
		HTMLElementName.LI,
		HTMLElementName.A,
		HTMLElementName.BIG,
		HTMLElementName.BLOCKQUOTE,
		HTMLElementName.ADDRESS,
		HTMLElementName.H1,
		HTMLElementName.H2,
		HTMLElementName.H3,
		HTMLElementName.H4,
		HTMLElementName.H5,
		HTMLElementName.H6,
		HTMLElementName.BODY};
	
	List<String> whiteList = Arrays.asList(w);
	
	//String [] e = {onCl
	
	//Additional regex for keywords missed during tag scrub
	Pattern pattern = Pattern.compile("(javascript)|" +
			"\\b[on][a-zA-Z]+[=]+", Pattern.CASE_INSENSITIVE);

	Pattern whitePattern = Pattern.compile("[\\s]{3,}|\n");

	//constructor
	htmlfilterScrubber(String outFile, String rawHtml) 
		throws MalformedURLException, IOException{
		writeHtml(outFile, rawHtml);
	}
	
	//get html
	String getHtml(String htmlFile) throws MalformedURLException, IOException{
		Source source=new Source(new URL(htmlFile));
		String renderedText=source.getRenderer().toString();
		//System.out.println("\nSimple rendering of the HTML document:\n");
		//System.out.println(renderedText);
		
		return renderedText;
	}
	
	String getHtml3(String htmlFile) throws MalformedURLException, IOException{
		MicrosoftConditionalCommentTagTypes.register();
		PHPTagTypes.register();
		// remove PHP short tags for this example otherwise
		//they override processing instructions
		PHPTagTypes.PHP_SHORT.deregister(); 
		MasonTagTypes.register();
		
		Source source=new Source(new URL(htmlFile));
		OutputDocument outputDocument=new OutputDocument(source);
		
		List<Tag> tags=source.getAllTags();
		//System.out.println(tags.size());
		  for (Tag tag : tags) {
			  if(tag.getElement() != null){
				if (whiteList.contains(tag.getElement().getName())) {
					if (tag.getElement().getName()=="a"){ 
						if (tag.getElement().getAttributes()!=null){ 
							String st = tag.getElement().getStartTag().tidy(true);
							int p = st.indexOf("\"");
							int end = st.indexOf("\"",p+1);
							st = st.substring(p,end);
							st = "<a href=" + st + ">";
							outputDocument.replace(tag.getElement().getStartTag()
									,st);
						}
					}
				} 
				else {
					outputDocument.remove(tag);
				}
			  }
		  }
		  
		 
		  return outputDocument.toString();
  }

	

	private static String getTitle(Source source) {
		Element titleElement=source.getFirstElement(HTMLElementName.TITLE);
		if (titleElement==null) return null;
		// TITLE element never contains other tags so just decode it collapsing whitespace:
		return CharacterReference.decodeCollapseWhiteSpace(titleElement.getContent());
	}

	private static String getMetaValue(Source source, String key) {
		for (int pos=0; pos<source.length();) {
			StartTag startTag=source.getNextStartTag(pos,"name",key,false);
			if (startTag==null) return null;
			if (startTag.getName()==HTMLElementName.META)
				return startTag.getAttributeValue("content"); // Attribute values are automatically decoded
			pos=startTag.getEnd();
		}
		return null;
	}

	public void writeHtml(String outFile, String rawText)
		throws MalformedURLException, IOException{
		//clean html
		String strippedText = getHtml3(rawText);
		
		//second regex scrub on remaining tags
		String cleanText = chkHtml(strippedText);
		  
		//write new html
		String header = "<HTML><HEAD>" +
				"<meta http-equiv=\"Content-Type\" content=\"text/html;" +
				"charset=UTF-8\" /><TITLE>Tepid eMail</TITLE></HEAD>";
		String newHtml = header + cleanText + "</HTML>";
		
		//write file
		try {
		    BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
		    out.write(newHtml);
		    out.close();
		} catch (IOException e) {
		}
	}

	//check valid html; invalid return no events or javascript
	//TODO: create new html
	//TODO: make makefile
	String chkHtml(String htmlText){
		Matcher matcher = pattern.matcher(htmlText);
		Matcher matcher2 = whitePattern.matcher(htmlText);
		
		while(matcher.find()){
			htmlText = htmlText.replaceAll(matcher.group(), "");	
		}
		
		while(matcher2.find()){
			htmlText = htmlText.replaceAll(matcher2.group(), "</br>");
		}
		
		return htmlText;
	}
}
